use crate::{
    error::{Error, Result},
    py_any_extras::PyAnyExtras,
};
use prost_reflect::{DynamicMessage, ReflectMessage, Value};
use pyo3::{PyAny, ToPyObject, PyObject};

pub fn merge_msg_into_pyobj(obj: &PyAny, msg: &DynamicMessage) -> Result<()> {
    for field in msg.descriptor().fields() {
        let field_name = field.name();
        let proto_meta = obj.get_proto_meta()?;
        if let Some(field_value) = msg.get_field_by_name(field_name) {
            obj.setattr(
                field_name,
                map_field_value(field_name, &field_value, proto_meta)?,
            )?;
        }
    }
    Ok(())
}

fn map_field_value(
    field_name: &str,
    field_value: &Value,
    proto_meta: &PyAny,
) -> Result<PyObject> {
    let py = proto_meta.py();
    match field_value {
        Value::Bool(x) => Ok(x.to_object(py)),
        Value::Bytes(x) => Ok(x.to_object(py)),
        Value::F32(x) => Ok(x.to_object(py)),
        Value::F64(x) => Ok(x.to_object(py)),
        Value::I32(x) => Ok(x.to_object(py)),
        Value::I64(x) => Ok(x.to_object(py)),
        Value::String(x) => Ok(x.to_object(py)),
        Value::U32(x) => Ok(x.to_object(py)),
        Value::U64(x) => Ok(x.to_object(py)),
        Value::Message(msg) => {
            let obj = proto_meta.create_instance(field_name)?;
            merge_msg_into_pyobj(obj, msg)?;
            Ok(obj.to_object(py))
        }
        value => Err(Error::UnsupportedType(value.to_string())),
    }
}
